package nmspack;

import java.net.*;
import java.io.*;
import org.w3c.dom.*;
import toolspack.*;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Stefano Ricciarelli
 * @version 1.0
 */

public class ApplicationPusher implements Runnable{

  private int rnid;
  private Document applicationXML;
  private int blockSize = 1024;
  private NMSConfigurator config = NMSConfigurator.getInstance();
  private static int myTcpTimeout = 1000;

  public ApplicationPusher(int rnid, Document applicationXML) {
    this.rnid           = rnid;
    this.applicationXML = applicationXML;
  }

  public void run() {
    Socket socket = null;
    try {
      Address rnAddr = config.getRnTcpAddress(rnid);
      if (rnAddr==null) {
        Logger.print("NMS: impossibile installare sul RN, rnid inesitente");
        return;
      }
      XmlTools xmlTools = XmlTools.getInstance();
      String applicationString = xmlTools.XML2string(applicationXML);
      if ( applicationString == null) {
        Logger.print(this, "ERRORE in applicationXML ");
        return;
      }

      socket = new Socket(rnAddr.getRemoteHost(), rnAddr.getRemotePort());
      socket.setSoTimeout(myTcpTimeout);
      OutputStream out    = socket.getOutputStream();
      InputStream  in     = socket.getInputStream();
      ObjectOutputStream outObj= new ObjectOutputStream(out);

      //   PROTOCOLLO RETI numero ******   4    *******
      //invio l'applicationXML
      outObj.writeObject(applicationString);

      Logger.print(this, "applicationXML inviato");
      //aspetto una risposta
      ObjectInputStream   inObj = new ObjectInputStream(in);
      String response = (String)inObj.readObject();
      if ( ! response.equals("OK") ) {
        Logger.print(this, "ERRORE il RN non ha accettato la richiesta ");
        return;
      }
      Logger.print(this, "applicationXML arrivato a destinazione, ora invio i files");
      //outObj.writeObject("ciao");
      //Logger.print(this, "spedito 'ciao'");
      //Logger.print(this, "arrivato: " + (String)inObj.readObject());
      Element root    = applicationXML.getDocumentElement();
      String basePath = root.getAttribute("basePath");
      int aid         = Integer.parseInt(root.getAttribute("aid"));

      File nmsAbsolutePath = new File(basePath);
      Logger.print(this, "inizio trasferimento file");
      if ( ! xCopy(nmsAbsolutePath, new File(""), outObj) ) {
        Logger.print(this, "ERRORE trasferendo i files TRASFERIMENTO ABORTITO ");
        return;
      }
      //invio il comando che indica la fine del trasferimento
      outObj.writeObject("end_copy");
      //attendo la conferma di tutto
      response = (String)inObj.readObject();
      if ( response.equals("OK") ) {
        Logger.print(this, "Trasferimento file terminato con successo e CONERMATO");
      } else {
        Logger.print(this, "ERRORE il RN non ha accettato la richiesta ");
        return;
      }
      outObj.close();
      inObj.close();
      socket.close();
      //aggiungo aid alla TableOfAid. Se gi presente questo invocazione non ha effetto
      config.addAid(aid);
      //aggiungo rnid alla TableOfAid
      if ( ! config.addRNforAID(rnid, aid) ) {Logger.print("ERRORE inserendo rnid nella table of aid"); return;}
      Logger.print(this, "ApplicationPusher termina con successo");
    } catch (Exception ex) {
      Logger.print(this, "ERRORE. Impossibile inviare la richiesta al RN. " + ex);
      try{socket.close();}catch (Exception e2){}
      return;
    }


  }

  private boolean xCopy(File file, File relativePath, ObjectOutputStream outObj) {
    try {
      if (file.isDirectory()) {
        if (! relativePath.getPath().equals("") ) {
          //Eseguito tutte le volte, tranne la prima invocazione
          Logger.print(this, "** invio directory... " + relativePath);
          outObj.writeObject("directory");
          outObj.writeObject(relativePath);
        }
        File[] files = file.listFiles();
        if (files.length==0) return true;
        boolean b = true;
        int i=0;
        while (i<files.length && b) {
          if (files[i].isDirectory())
            b = b && xCopy(files[i], new File(relativePath, files[i].getName()), outObj);
          else b = b && xCopy(files[i], relativePath, outObj);
          i++;
        }
        return b;
      } else {
        //invio false seguito dal nome senza percorso del file
        File relativeFile = new File(relativePath, file.getName());
        Logger.print(this, "** invio file... " + relativeFile);
        outObj.writeObject("file");
        outObj.writeObject(relativeFile);
        FileInputStream file_in = new FileInputStream(file);
        //invio il file
        Files.copy(file_in, outObj, blockSize);
        file_in.close();
        return true;
      }
    } catch (Exception ex) {
      Logger.print(this, "ERRORE trasferendo il file - TRASFERIMENTO ABORTITO " + ex);
      return false;
    }
  }


/*  private void writeAnObject (OutputStream out, Object obj) throws Exception {
    ObjectOutputStream o = new ObjectOutputStream(out);
    o.writeObject(obj);
    o.close();
  }

  private Object readAnObject (InputStream in) throws Exception {
    ObjectInputStream o = new ObjectInputStream(in);
    Object obj = o.readObject();
    o.close();
    return obj;
  }*/


 /* private boolean xCopy(File file, OutputStream so_out, ObjectOutputStream so_outObj) {
    try {
      if (file.isDirectory()) {
        //invio true seguito dal nome senza percorso della directory
        Logger.print(this, "** invio directory...");
        so_outObj.writeObject("new_directory");
        so_outObj.writeObject(file.getName());
        Logger.print(this, "** ... " + file);
        File[] files = file.listFiles();
        if (files.length==0) return true;
        boolean b = true;
        int i=0;
        while (i<files.length && b) {
          b = b && xCopy(files[i], so_out, so_outObj);
          i++;
        }
        so_outObj.writeObject("end_directory");
        return b;
      } else {
        //invio false seguito dal nome senza percorso del file
        Logger.print(this, "** invio file... ");
        so_outObj.writeObject("file");
        so_outObj.writeObject(file.getName());
        Logger.print(this, "** ... " + file);
        FileInputStream file_in = new FileInputStream(file);
        //invio il file
        Files.copy(file_in, so_out, blockSize);
        file_in.close();
        return true;
      }
    } catch (Exception ex) {
      Logger.print(this, "ERRORE trasferendo il file - TRASFERIMENTO ABORTITO " + ex);
      return false;
    }
  }*/


}//fine classe

 /*   //fase di ricezione richieste:
    String fileName = null;
    try {
      String basePath = applicationXML.getDocumentElement().getAttribute("basePath");
      InputStream in = socket.getInputStream();
      ObjectInputStream inObj = new ObjectInputStream(in);


      do {
        fileName = (String)inObj.readObject();
        if ( ! fileName.equals("///") ) {
          FileInputStream fileIn = new FileInputStream(new File(basePath, fileName));
          Files.copy(fileIn, out, blockSize);
          fileIn.close();
          Logger.print(this, "File " + fileName + " trasferito con sucesso!");
        }
      } while ( ! fileName.equals("///") );
    } catch (Exception ex) {
      Logger.print(this, "ERRORE trasferendo il file " + fileName + " TRASFERIMENTO ABORTITO " + ex);
      return;
    }*/