package nmspack;

import javax.xml.parsers.*;
import toolspack.*;
import java.util.*;
import java.net.Socket;
import java.io.*;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Stefano Ricciarelli
 * @version 1.0
 */

public class NMSConfigurator {

  private static NMSConfigurator instance = new NMSConfigurator();
  private DocumentBuilder builder;
  //NB: il funzionamento delle seguenti Map  solo per VALORE. Un'eventuale structure-sharing fra gli oggetti presenti nelle varie
  //tabelle non ha alcuna influenza
  private Map tableOfWS;  // Integer    wsid   - Integer            aid
  private Map tableOfRN;  // Integer    rnid   - Address            address
  private Map tableOfAID; // Integer    aid    - TreeSet of Integer rnid
  private Map tableOfAB;  // String     abName - Integer            rnid
  private int rnid = 0;

  private static final int tableOfABcode  = 1;
  private static final int tableOfAIDcode = 2;
  private static final int tableOfRNcode  = 3;
  private static final int tableOfWScode  = 4;
  private static final int rnidcode       = 5;

  private int nms2nmsTcpPort;
  private Address nms2Address;
  private Socket nms2nmsSocket;
  private ObjectInputStream  ois;
  private ObjectOutputStream oos;
  private int nms2nmsTimeout;

  private boolean active;            //indica se  il NMS attivo
  private boolean copying = false;   //indica se sta copiando i dati sull'altro NMS (non bisogna fare modifiche nel frattempo!!)
  private boolean synch   = false;   //indica se  presente una copia passiva sincronizzata

  //contatore globale. Utilizzato dal servizio udp GetRNService per eseguire il roundrobin sui RN
  private int staticCounter = -1; //il primo valore restituito  0

  /**
   * Il costruttore  privato poich si tratta di una classe singleton
   */
  private NMSConfigurator() {
    DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
    tableOfWS  = new TreeMap();
    tableOfRN  = new TreeMap();
    tableOfAID = new TreeMap();
    tableOfAB  = new TreeMap();
  /*  tableOfWS  = Collections.synchronizedMap(new HashMap());
    tableOfRN  = Collections.synchronizedMap(new HashMap());
    tableOfAID = Collections.synchronizedMap(new HashMap());
    tableOfAB  = Collections.synchronizedMap(new HashMap());*/
    try { builder = dbFactory.newDocumentBuilder();
    }catch (Exception ex) {Logger.print(this, "errore nell'inizializzazione di NMSConfigurator" + ex); }
  }

  /**
   * Metodo statico da invocare per ottenere il riferimeto all'unico oggetto esistente
   * di questa classe
   * @return il riferimento all'unico oggetto di questa classe
   */
  public static NMSConfigurator getInstance() {
    return instance;
  }

  //MODIFICA staticConter
  public synchronized int getNewStaticCounterValue() {
    staticCounter++;
    return staticCounter;
  }
  //MODIFICA staticConter
  public synchronized void resetStaticCounterValue() {
    staticCounter = -1; //il primo valore restituito  0
  }
  //MODIFICA rnid
  public synchronized int getNewRnid() {
    rnid++;
    updateRemoteRnid(rnid, rnidcode);
    return rnid;
  }
  //READ-ONLY
  public synchronized int getRnid() {
    return rnid;
  }
  //MODIFICA tableOfWS
  public synchronized void addWS (int wsid) {
    //nota: non inserisco null come aid perch altrimenti in successive ricerche con get() non sarei in grado di discriminare
    //se si tratta di una chiave wsid inesistente, oppure di un wsid senza aid associato
    tableOfWS.put(new Integer(wsid), new Integer(-1));
    updateRemoteTable(tableOfWS, tableOfWScode);
  }
  //MODIFICA tableOfWS
  public synchronized boolean mapWStoAID (int wsid, int aid) {
    Object o = tableOfWS.get(new Integer(wsid));
    if (o==null) return false;  //wsid inesistente
    tableOfWS.put(new Integer(wsid), new Integer(aid));
    updateRemoteTable(tableOfWS, tableOfWScode);
    return true;
  }
  //READ-ONLY
  public synchronized int getAIDofWSID (int wsid) {
    Integer aid = (Integer)tableOfWS.get(new Integer(wsid));
    if (aid==null) return -1;
    return aid.intValue(); //vale -1 se non  presente alcun aid
  }
  //MODIFICA tableOfRN
  public synchronized void addRN (int rnid, Address addr) {
    tableOfRN.put(new Integer(rnid), addr);
    updateRemoteTable(tableOfRN, tableOfRNcode);
  }
  //MODIFICA tableOfRN, tableOfAB e tableOfAID
  //Per mantenere consistenza alla rimozione di un RN deve fare seguito la rimozione di tutti gli AB presenti su di esso
  //Ma devo anche rimuoverlo da tutti i TreeSet dei TableOfAID!!!!!!!! --> decido di gestire questa inconsistenza in lettura dalla
  //tableOfAid controllando che il RN esista veramente!!! (soluzione non bella)
  public synchronized void removeRN (int rnid) {
    Integer rn = new Integer(rnid);
    tableOfRN.remove(rn);
    Vector v = new Vector();
    if ( tableOfAB.containsValue(rn) ) {
      Logger.print(this, "Sono presenti degli AB installati sul RN da rimuovere, provvedo alla loro rimozione ");
      Iterator keys = tableOfAB.keySet().iterator();
      while (keys.hasNext()) {
        Object key = keys.next();
        //memorizzo tutte le chiavi da eliminare perch non posso eliminarle direttamente mentre scorro un Iterator!!!
        if ( tableOfAB.get(key).equals(rn) )  v.add(key);
      }//while
      for (int i=0; i<v.size(); i++) {
        tableOfAB.remove(v.elementAt(i));
        Logger.print(this, "rimosso lo AB " + v.elementAt(i));
      }//for
    }//if
    //procedo ad una eventuale rimozione all'interno di table of AID:
    Iterator keys2 = tableOfAID.keySet().iterator();
    while (keys2.hasNext()) {
      Set value = (Set)tableOfAID.get(keys2.next()); //estraggo il Set di RN per ciascun AID
      value.remove(rn); //rimuovo il RN dal set. (c'e strutur sharing per cui e' rimosso anche da dentro la Map)
    }
    updateRemoteTable(tableOfRN,  tableOfRNcode);
    updateRemoteTable(tableOfAB,  tableOfABcode);
    updateRemoteTable(tableOfAID, tableOfAIDcode);
  }
/*  private void checkTableOfAID(Integer rn) {
    int length = tableOfAID.size();
    Iterator keys = tableOfAID.keySet().iterator();
    while (keys.hasNext()) {
      Set value = (Set)tableOfAID.get(keys.next()); //estraggo il Set di RN per ciascun AID
      value.remove(rn); //rimuovo il RN dal set. (c'e strutur sharing per cui e' rimosso anche da dentro la Map)
    }
  }*/
  //READ-ONLY
  //devo risolvere l'associazione inversa, ossia dato il valore trovare la chiave
  public synchronized int getRnidFromRnAddress (Address addr) {
    if ( tableOfRN.containsValue(addr) ) {
      //Logger.print(this, "getRnidFromRnAddress:  esiste questa chiave!! ");
      Iterator keys = tableOfRN.keySet().iterator();
      while (keys.hasNext()) {
        Object key = keys.next();
        if ( tableOfRN.get(key).equals(addr) ) return ((Integer)key).intValue();
      }//while
    }//if
    //Logger.print(this, "getRnidFromRnAddress:  NON esiste questa chiave!! ");
    return -1;
  }
  //READ-ONLY
  public synchronized Address getRnTcpAddress (int rnid) {
    return (Address)tableOfRN.get(new Integer(rnid));
  }
  //READ-ONLY
  public synchronized Address[] getRegisteredRnForAid(int aid) {
    //Logger.print(this, "getRegisteredRnForAid...");
    Set rnList = (Set)tableOfAID.get(new Integer(aid));
    if (rnList==null) return null;
    Object[] rnids = rnList.toArray();
    int length = rnids.length;
    Address[] addr = new Address[length];
    for (int i=0; i<length; i++)
      addr[i] = (Address)tableOfRN.get(rnids[i]);
    //Logger.print(this, "getRegisteredRnForAid OK");
    return addr;
  }
  //MODIFICA tableOfAB
  public synchronized void addAB (String abName, int rnid) {
    tableOfAB.put(abName, new Integer(rnid));
    updateRemoteTable(tableOfAB, tableOfABcode);
  }
  //READ-ONLY
  public synchronized Address lookupAB (String abName) {
    Integer i = null;
    i = (Integer)tableOfAB.get(abName);
    if (i == null) return null;
    int rnid = i.intValue();
    return getRnTcpAddress(rnid);
  }
  //MODIFICA tableOfAID
  public synchronized void addAid(int aid) {
    if (! existAid(aid) )  tableOfAID.put(new Integer(aid), new TreeSet());
    updateRemoteTable(tableOfAID, tableOfAIDcode);
  }
  //READ-ONLY
  public synchronized boolean existAid(int aid) {
    return tableOfAID.containsKey(new Integer(aid));
  }
  //MODIFICA tableOfAID
  public synchronized boolean addRNforAID (int rnid, int aid) {
    Logger.print(this, "addRNforAID ...");
    Set rnList = (Set)tableOfAID.get(new Integer(aid));
    if (rnList==null) return false;
    rnList.add(new Integer(rnid));
    tableOfAID.put(new Integer(aid), rnList);
    Logger.print(this, "addRNforAID OK");
    updateRemoteTable(tableOfAID, tableOfAIDcode);
    return true;
  }

  //MODIFICHE EFFETTUABILI SOLO IN MODALITA NOT-ACTIVE:
  public synchronized void setTableOfRN (Map table) {
    if (active) { Logger.print(this, "ERRORE, impossibile invocare questa funzionalit in modalit attiva"); return;}
    Logger.print(this, "**** Sto scrivendo tableOfRN di  " + table.size() + " elementi");
    this.tableOfRN = table;
  }
  public synchronized void setTableOfWS (Map table) {
    if (active) { Logger.print(this, "ERRORE, impossibile invocare questa funzionalit in modalit attiva"); return;}
    Logger.print(this, "**** Sto scrivendo tableOfWS di  " + table.size() + " elementi");
    this.tableOfWS = table;
  }
  public synchronized void setTableOfAID (Map table) {
    if (active) { Logger.print(this, "ERRORE, impossibile invocare questa funzionalit in modalit attiva"); return;}
    Logger.print(this, "**** Sto scrivendo tableOfAID di  " + table.size() + " elementi");
    this.tableOfAID = table;
  }
  public synchronized void setTableOfAB (Map table) {
    if (active) { Logger.print(this, "ERRORE, impossibile invocare questa funzionalit in modalit attiva"); return;}
    Logger.print(this, "**** Sto scrivendo tableOfAB di  " + table.size() + " elementi");
    this.tableOfAB = table;
  }
  public synchronized void setRnid(int rnid) {
    if (active) { Logger.print(this, "ERRORE, impossibile invocare questa funzionalit in modalit attiva"); return;}
    this.rnid = rnid;
  }

  //PRIMA DI CHIAMRE LE SEGUENTI OPERAZIONI PASSARE ALLA MODALITA COPYING
  public synchronized Map getTableOfRN () {
    return tableOfRN;
  }
  public synchronized Map getTableOfWS () {
    return tableOfWS;
  }
  public synchronized Map getTableOfAID () {
    return tableOfAID;
  }
  public synchronized Map getTableOfAB () {
    return tableOfAB;
  }

  //METODI DI UPDATE
  private synchronized void updateRemoteTable (Map table, int opCode) {
    if (! synch) return; //ritorno true nel caso in cui non debbo fare niente
    try {
      oos.writeObject(new Integer(opCode));
      Logger.print(this, "NMS attivo: sto scrivendo sul collegamento col passivo una mappa di " + table.size() + " elementi");
      oos.writeObject(table);
      oos.flush();
      //attendo una conferma
      //if ( ! ois.readBoolean() ) {
      if ( ! ((Boolean)ois.readObject()).booleanValue() ) {
        Logger.print(this, "NMS attivo: errrore restituito dal NMS passivo: mi setto come unico NMS ");
        setAlone();
      }//if
    } catch (Exception ex) {
      Logger.print(this, "ERRORE cercando di sincronizzare la copia passiva: mi setto come unico NMS  " + ex);
      setAlone();
    }//catch
    Logger.print(this, "NMS attivo: aggiornato correttamente il NMS passivo con una mappa di " + table.size() + "elementi e opcode = " + opCode);
  }
  private synchronized void updateRemoteRnid (int rnid, int opCode) {
    if (! synch) return; //ritorno nel caso in cui non debbo fare niente
    try {
      oos.writeObject(new Integer(opCode)); //oos.writeInt(opCode);
      oos.writeObject(String.valueOf(rnid));//oos.writeInt(rnid);
      oos.flush();
      //attendo una conferma
      if ( ! ((Boolean)ois.readObject()).booleanValue() ) {
        Logger.print(this, "NMS attivo: errrore restituito dal NMS passivo: mi setto come unico NMS ");
        setAlone();
      }//if
    } catch (Exception ex) {
      Logger.print(this, "ERRORE cercando di sincronizzare la copia passiva: mi setto come unico NMS " + ex);
      setAlone();
    }//catch
    Logger.print(this, "NMS attivo: aggiornato correttamente il NMS passivo ");
  }

  public synchronized void setAlone () {
    try{ois.close();   }catch(Exception e2){}
    try{oos.close();   }catch(Exception e2){}
    try{nms2nmsSocket.close();}catch(Exception e2){}
    ois    = null;
    oos    = null;
    nms2nmsSocket = null;
    synch  = false;
  }

  //************** bean properties **********************//
  public synchronized int getNms2nmsTcpPort () {
    return nms2nmsTcpPort;
  }
  public synchronized void setNms2nmsTcpPort (int nmsPort) {
    this.nms2nmsTcpPort = nmsPort;
  }
  public synchronized int getNms2nmsTimeout () {
    return nms2nmsTimeout;
  }
  public synchronized void setNms2nmsTimeout (int nms2nmsTimeout) {
    this.nms2nmsTimeout = nms2nmsTimeout;
  }
  public synchronized Address getNms2Address () {
    return nms2Address;
  }
  public synchronized void setNms2Address (Address nms2Address) {
    this.nms2Address = nms2Address;
  }
  public synchronized boolean isActive () {
    return active;
  }
  public synchronized void setActive (boolean active) {
    this.active = active;
  }
  public synchronized boolean isCopying () {
    return copying;
  }
  public synchronized void setCopying (boolean copying) {
    this.copying = copying;
  }
  public synchronized boolean isSync () {
    return synch;
  }
  public synchronized void setSynch (boolean synch) {
    this.synch = synch;
  }
  public synchronized Socket getNms2nmsSocket () {
    return nms2nmsSocket;
  }
  public synchronized void setNms2nmsSocket (Socket socket) {
    this.nms2nmsSocket = socket;
  }
  public synchronized ObjectOutputStream getOos () {
    return oos;
  }
  public synchronized void setOos (ObjectOutputStream oos) {
    this.oos = oos;
  }
  public synchronized ObjectInputStream getOis () {
    return ois;
  }
  public synchronized void setOis (ObjectInputStream ois) {
    this.ois = ois;
  }
}//fine classe
