package nmspack;

import toolspack.*;
import java.util.Map;
import java.net.*;
import java.io.*;
import org.w3c.dom.*;
import toolspack.udppack.*;
import nmspack.udpservices.*;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Stefano Ricciarelli
 * @version 1.0
 */

public class StartNMS {

  private String logPath = "c:/dev/log/";

  private InetAddress myAddress;
  private InetAddress nms2Address;

  //dati inerenti la porta UDP utilizzata per tutte le comunicazioni in ricezione da RN e WS
  private short fixedMsgLength      = 1024;
  private int   myUDPPort           = 4000;

  //dati inerenti la porta TCP utilizzata per le comunicazioni in ricezione dal secondo NMS
  private int    nms2nmsTcpPort      = 4002; //NB la porta  la stessa per entrambi
  private String nms2HostName;  //        = "169.254.0.2";
  private int nms2nmsTimeout = 3000;

  private short rnRegisterServiceId = 1;
  private short abRegisterServiceId = 2;
  private short abLookupServiceId   = 3;
  private short wsRegisterServiceId = 10;
  private short getRnServiceId      = 11;
  private short removeRnServiceId   = 20;
  //private short updateNnmServiceId  = 12;

  private NMSConfigurator config = NMSConfigurator.getInstance();


  public StartNMS() {
  }
  public static void main(String[] args) {
    StartNMS start = new StartNMS();
    start.exec(args);
  }

  public synchronized void exec(String[] args) {
    new Logger(logPath);
    if (args.length != 1) {
      Logger.print(this, "ERRORE negli argomenti. NMS TERMINATO");
      return;
    }//if
    nms2HostName = args[0];

    //NMSConfigurator config = NMSConfigurator.getInstance();
    //FASE 1 - INIZIALIZZAZIONE
    try {
      config.setNms2nmsTcpPort(nms2nmsTcpPort);
      config.setNms2nmsTimeout(nms2nmsTimeout);
      myAddress = InetAddress.getLocalHost();
      nms2Address = InetAddress.getByName(nms2HostName);
      Address addr = new Address(nms2Address, nms2nmsTcpPort);
      Logger.print(this, "Secondo NMS impostato: " + addr);
      config.setNms2Address(addr);
      Logger.print(this, "NMS in esecuzione su: " + myAddress);
      IUDPServiceSocket udpSocket = new UDPServiceSocket(myAddress, myUDPPort, fixedMsgLength);
      udpSocket.registerService(rnRegisterServiceId, new RegisterRNService());
      udpSocket.registerService(abRegisterServiceId, new RegisterABService());
      udpSocket.registerService(abLookupServiceId,   new LookupABService());
      udpSocket.registerService(wsRegisterServiceId, new RegisterWSService());
      udpSocket.registerService(getRnServiceId     , new GetRNService());
      udpSocket.registerService(removeRnServiceId  , new RemoveRNService());
      //udpSocket.registerService(updateNnmServiceId , new UpdateNMSService());
      //inizializzo la replicazione del NMS
      //NMSReplicationStarter replic = new NMSReplicationStarter();
      if (! tryConnect() ) {
        Logger.print(this, "ERRORE inizializzando la replicazione del NMS. NMS TERMINATO");
        return;
      }//if
    }
    catch (Exception ex) {
      Logger.print(this, "ERRORE inizializzando i servizi di risposta ai messaggi UDP " + ex + " NMS TERMINATO");
      return;
    }
    if (config.isActive()) Logger.print(this, "NMS installato correttamente in modalit attiva");
    else Logger.print(this, "NMS installato correttamente in modalit passiva");
    //FASE 2 - ASCOLTO DEI COMANDI
    //read_data:
    while(true) {
      //istruzione bloccante:
      String command = Console.readLine("NMS command --> ");
      if (! config.isActive() ) {
        Logger.print(this, "Impossibile accettare comandi, questa copia di NMS  passiva");
        break;
      }
      String[] s = command.split("\\s");
      if (s.length <= 2) Logger.print("NMS: comando sconosciuto ");
      else {
        if (s.length != 3) {
          Logger.print("NMS: comando errato; sintassi 'map wsid aid'");
          return;
        }
        Logger.print("Command " + s[0] + ", param " + s[1] + ", " + s[2]);
        if (s[0].equalsIgnoreCase("installOnRN")) installOnRN(s);
        if (s[0].equalsIgnoreCase("map")) map(s);
      }
    }//while true;
  }

  private synchronized void installOnRN(String[] s) {
    try {
      String fileName = s[1];
      String rnid     = s[2];
      XmlTools xmlTools = XmlTools.getInstance();
      Document doc = xmlTools.parseXMLFile(fileName);
      ApplicationPusher ap = new ApplicationPusher(Integer.parseInt(rnid), doc); // rnAddr.getRemoteHost(), rnAddr.getRemotePort() ,doc);
      Thread t2 = new Thread(ap);
      t2.start();
    } catch (Exception ex) {
      Logger.print("START NMS: ERRORE " + ex);
    }
  }

  private synchronized void map(String[] s) {
    //map wsid aid
    boolean ok = config.mapWStoAID(Integer.parseInt(s[1]), Integer.parseInt(s[2]));
    if (! ok) Logger.print(this, "ERRORE mappando l'applicazione sul WS");
    else Logger.print(this, "Mappatura effettuata");
  }

  public boolean tryConnect() {
    try {
      Thread t1 = null;
      Thread t2 = null;
      Address nms2Address = config.getNms2Address();
      Socket nms2nmsSocket = new Socket(nms2Address.getRemoteHost(), nms2Address.getRemotePort());
      //se la connessione  andata a buon fine devo acquisire tutte le tabelle. Contemporaneamente l'altro NMS
      //deve inviare tutti questi dati e bloccare momentaneamente tutte le operazioni di modifica ti tali dati
      //(non  troppo vincolante poich le operazioni di modifica sono veramente rare, ad eccezione del getNewRnid
      //che sarebbe meglio non bloccare)
      try {
        // ****      PASSIVO      **** //
        nms2nmsSocket.setSoTimeout(config.getNms2nmsTimeout());
        nms2nmsSocket.setTcpNoDelay(true);
        nms2nmsSocket.setKeepAlive(true);
        //la seguente  bloccante
        ObjectInputStream ois  = new ObjectInputStream(nms2nmsSocket.getInputStream());
        ObjectOutputStream oos = new ObjectOutputStream(nms2nmsSocket.getOutputStream());
        config.setNms2nmsSocket(nms2nmsSocket);
        config.setOis(ois);
        config.setOos(oos);
        Logger.print(this, "NMS passivo:  connessione accettata - Mi imposto come NMS passivo e mi sincronizzo");
        Logger.print(this, "NMS passivo:  Impostazione del keepalive della socket " + nms2nmsSocket.getKeepAlive());
        config.setActive(false);
        config.setCopying(true);
        config.setSynch(false);

        Logger.print(this, "NMS passivo:  ois aperto");
        config.setTableOfAB ((Map)ois.readObject());
        config.setTableOfAID((Map)ois.readObject());
        config.setTableOfRN ((Map)ois.readObject());
        config.setTableOfWS ((Map)ois.readObject());
        //config.setRnid      (     ois.readInt());
        config.setRnid( ((Integer)ois.readObject()).intValue() );
        //ATTENZIONE:  logicamente sbagliato dare l'ok prima di avere finito, ma sono costretto altrimenti c' un conflitto col
        //SynchronizingThread che legge e scrive sugli stessi ois e oos. Il problema non  grave perch al max NMS attivo decider di proseguire
        //da solo, e quindi questo passivo se ne accorger e tenter di riconnettersi
        oos.writeObject("ok");
        oos.flush();

        nms2nmsSocket.setSoTimeout(0); //nessun timeout
        Logger.print(this, "NMS passivo:  Dati ricevuti correttamente. Lancio il SynchronizingThread.");
        t1 = new Thread( new SynchronizingThread() );
        t1.start();
        config.setSynch(true);
        Logger.print(this, "NMS passivo:  lancio il thread ConnectionListener per diventare attivo se il NMS attivo cade ");
        t2 = new Thread( new ConnectionListener() );
        t2.start();
        Logger.print(this, "NMS passivo:  oos aperto. Invio la conferma al NMS attivo");
        //oos.writeUTF("ok");
    /*    oos.writeObject("ok");
        oos.flush(); */
        Logger.print(this, "NMS passivo:  conferma inivata");
        Logger.print(this, "NMS passivo:  La socket  chiusa? " + nms2nmsSocket.isClosed());
    /*    Logger.print(this, "NMS passivo:  Dati ricevuti correttamente. Lancio il SynchronizingThread.");
        Thread t = new Thread( new SynchronizingThread() );
        t.start();
        config.setSynch(true);*/
        config.setCopying(false);
        return true;
      } catch (Exception ex) {
        Logger.print(this, "NMS passivo:  ERRORE FATALE - Terminare questo NMS " + ex);
        try{t1.interrupt();}catch (Exception e1){}
        try{t2.interrupt();}catch (Exception e1){}
        config.setAlone();
        return false;
      }//catch interno
    } catch (Exception ex) {

      // ****     ATTIVO      **** //
      Logger.print(this, "connessione Fallita - Mi imposto come NMS attivo e lancio la socket di ascolto");
      config.setActive(true);
      Thread t = null;
      try {
        Logger.print(this, "NMS attivo:  lancio il ServerSocket per accettare richieste di sincornizzazione da parte del NMS passivo");
        t = new Thread( new NMSServerSocket() );
        t.start();
        return true;
      } catch (Exception ex2) {
        Logger.print(this, "NMS attivo: ERRORE lanciando il thread " + ex + " - Terminare questo NMS ");
        try{t.interrupt();}catch (Exception e1){}
        return false;
      }//catch interno
    }//catch esterno
  }

}