package nmspack.udpservices;

import toolspack.udppack.IService;
import toolspack.udppack.IServiceEndPoint;
import java.io.*;
import toolspack.*;
import nmspack.*;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Stefano Ricciarelli
 * @version 1.0
 */

public class RemoveRNService implements IService {


  private IServiceEndPoint sep;
  private byte[] request;
  private NMSConfigurator config;

  public RemoveRNService() {
    config = NMSConfigurator.getInstance();
  }
  public void setServiceEndPoint(IServiceEndPoint sep) {
    this.sep = sep;
  }
  public void setRequest(byte[] request) {
    this.request = request;
  }
  /**
   * Quando pu essere richiesta la rimozione di un RN: in tutti quei casi in cui si interroga un RN e non si riceve alcuna risposta.
   * Precisamente: 1)installando un applicazione 2)richiedendo l'esecuzione di una nuova o vecchia sessione 3)richiedendo l'esecuzione di
   * di un AB remoto. Si noti che tutti i casi riguardano comunicazioni TCP. Nel caso 1) sono gi sul NMS quindi non importa richiamare
   * questo servizio UDP ma basta agire direttamente sul configurator. Sono da considerare allora solo i casi 2 e 3
   */
  public void run() {
    //PROTOCOLLO RETI numero  ** 7 ** - RIMOZIONE PRESSO NMS DI UN RN
    //ESECUZIONE one-shot. Finite le operazioni per questa richiesta termino.
    //Successive richieste genereranno altri thread
    if (config.isCopying() || !config.isActive()) {
      Logger.print(this, "Richiesta ignorata - questo NMS  passivo oppure  in fase di copia" );
      return;
    }
    try {
      DataInputStream din = new DataInputStream( new ByteArrayInputStream(request) );
      //Logger.print(this, "Servizio RemoveRN ");
      short iplength = din.readShort();
      byte[] ipraw   = new byte[iplength];
                       din.read(ipraw, 0, iplength);
      int port       = din.readInt();
      //addr  l'indirizzo del RN da eliminare. Nota che il fatto che la porta TCP sia multiplexata fa si che l'indirizzo di RN sia lo
      //stesso per un altro RN e per un WS. Se invece WS e RN accedessero agli altri RN attraverso porte diverse dovrei separare i due
      //casi per capire di quale RN  richiesta la rimozione
      Address addr   = new Address(ipraw, port);
      Logger.print(this, " ***** Servizio RemoveRN: si richiede di rimuovere " + addr);
      //ottengo l'rnid di quell'indirizzo
      int rnid       = config.getRnidFromRnAddress(addr); //cerco se per caso  gi stato registrato questo RN
      if (rnid == -1) {
        Logger.print(this, "ATTENZIONE: Il RN di cui  richiesta l'eliminazione non esiste");
        return;
      } else {
        config.removeRN(rnid);
        rnid = config.getNewRnid();
        Logger.print(this, "Rimozione del RN di indirizzo " + addr + " effettuata con successo");
      }//else
    } catch (Exception ex) {
      Logger.print(this, "ERRORE in removeRN " + ex);
      return;
    }//catch
  }



}//fine classe